function [I Q bits_rx]= Decoder_OFDM(y_real,y_imag,BitRate,SampleRate,TimeWindow,BpS,Nc,N_FFT,CP,Coder)

disp('Inicio Funcion Decoder_OFDM-------------------------------------')	

% Decoder_OFDM : Decodificador de seal electrica OFDM Cyclic Prefix y Zero Pading

%%% PARAMETROS DE ENTRADA
    % y_real: seal temporal real de entrada
    % y_imag: seal temporal imaginaria de entrada
    % TimeWindow: Ventana temporal de simulacion
    % BitRate: Tasa de bits del sistema
    % SampleRate: Tasa de muestreo del sistema
    % BpS: N bits por smbolo del mapeado de subportadoras de datos
    % N_FFT: N total de subportadoras y/o tamao de la FFT
    % Nc: N de subportadoras que transportan datos
    % CP: Longitud intervalo de guarda para Cyclix prefix o Zero Pading respecto el tamo de la FFT
    % Coder: - 1 Cyclix prefix-Zero Forcing
    %        - 2 Zero Pading-Zero Forcing Min Norm
    %        - 3 Zero Pading-Fast Zero Forcing 
%%% PARAMETROS DE SALIDA
    % I: Parte real de simbolos M-QAM de informacion recibidos
    % Q: Parte imaginaria de simbolos M-QAM de informacion recibidos
    % bits_rx: Secuencia de bits decodificados

    
%%%%SEAL RECIBIDA: FAASE Y QUADRATURA
%Desplazamientyo circular que compensa retardo negativo de dispesion de la
%fibra
T1 = TimeWindow-1.4e-9; %L = 1000 Km y D = 17 pseg/(Kmnm)
N1 = floor((T1/TimeWindow-floor(T1/TimeWindow))*TimeWindow*SampleRate);
signal_OFDM = complex(y_real,y_imag);
signal_OFDM = signal_OFDM(:);
signal_OFDM = [signal_OFDM(N1+1:end); signal_OFDM(1:N1)]; 

signal_OFDM = signal_OFDM(1:floor(SampleRate/(BitRate/BpS)):end);

% Clculo de la secuencia de bits efectiva transmitida. BitRateEfectivo=BitRate/(1+CP)
NTS_OFDM=floor(length(signal_OFDM )/(ceil(N_FFT*(1+CP)))); %N total de simbolos OFDM transmitidos
NTS_INFO=Nc*NTS_OFDM; %N total de simbolos de datos M-QAM transmmitidos
NTB_INFO=NTS_INFO*BpS; %N total de bits de datos transmitidos
NTS_ZP=(N_FFT-Nc)*NTS_OFDM; %N total de smbolos M-QAM equivalentes a portadoras de guarda
NTS_CP=NTS_OFDM*ceil(N_FFT*CP); %N total de smbolos M-QAM equivalentes a intervalo de guarda
NTS_casados=NTS_INFO+NTS_ZP+NTS_ZP; %N total de simbolos M-QAM transmitidos

% Extraccin de secuencia de ceros que compensan perdida de Bit Rate  efectivo
signal_OFDM = signal_OFDM (1:NTS_OFDM*ceil(N_FFT*(1+CP))); 
signal_OFDM = signal_OFDM(:);



%%%%DECODIFICADOR OFDM 
switch Coder
    
    case 1  %Cyclix prefix-Igualador Zero Forcing
        disp('EL modo de codificacion es CYCLIC PREFIX')
        signal_OFDM  = reshape(signal_OFDM ,ceil(N_FFT*(1+CP)),NTS_OFDM);
        % Extraccin de  Cyclic Prefix
        signal_OFDM  = signal_OFDM ((ceil(N_FFT*CP)+1):size(signal_OFDM ,1),:);
        % Decodificacion FFT
        simb_OFDM  = fft(signal_OFDM ,N_FFT)/sqrt(N_FFT); 
        % Estima de Canal Cyclic prefix M=64 CP=0.25 1024*8*32 bits T1 = TimeWindow-1.4e-9;
        PauCoefFase = [0.977537937014267	0.293057312059113	-0.386290009112565	-1.06016654911659	-1.72903442246315	-2.3917953878244	-3.05002065670353	-3.70379922675357	-4.35239330789767	-4.99359394744445	-5.63174539158081	-6.2630169350136	-6.8908512039122	-7.51258808689254	-8.12976957453346	-8.74058465611737	-9.34695414256154	-9.94765847773349	-10.5423410270717	-11.133543691787	-11.7182846260611	-12.297390534365	-12.8718705845407	-13.4413193149655	-14.0060137984606	-14.5648722308391	-15.1152352372114	-15.6314922539931	-16.110504623855	-16.5779781687335	-17.0865641234605	-17.6684896949884	-18.3258027080841	-19.0652827179314	-19.8749314012717	-20.7426987281563	-21.6349303369977	-22.5143406584817	-23.3620144186276	-24.1884628778156	-25.0028735672232	-25.8128805973177	-26.6193254816489	-27.4192120882731	-28.2130787356979	-29.0013149951696	-29.7861159568456	-30.5632195983107	-31.3365830377323	-32.1035707390764	-32.8696865064838	-33.6258084628581	-34.3770155817204	-35.1243948764156	-35.8687053352219	-36.6046474425748	-37.3361994665796	-38.0657000231446	-38.787121259967	-39.5022345389962	-40.2141345674272	-40.9201612122482	-41.619514506482	-42.3154101317378];       
        PauCoefMod = [0.000742314862027191	0.000740842721713987	0.000738843555125272	0.000736111874119678	0.00073393567392471	0.000732012214533824	0.000730457458526065	0.000728393167792909	0.000726603308505937	0.000723393418632084	0.000722090850295735	0.000719735667799619	0.000718471404157664	0.000715558846238007	0.000713432267777665	0.000711936426228359	0.000709303822214449	0.000706754166360795	0.000703661543948799	0.000702188280789723	0.000698821756957788	0.000696667579843221	0.000694026886858881	0.000692019152662812	0.000688757119876803	0.000686787952255977	0.000681078898724537	0.000659093489829768	0.00063944424999716	0.000642103483015186	0.000666064024033053	0.0007005266249234	0.000722159126686136	0.000725025560758782	0.000714998769783838	0.000714463133077979	0.000726689915940471	0.000751058484376376	0.000774299413259266	0.000776210725598525	0.000776699435809379	0.000774606830568099	0.000774262055603571	0.000772820535593445	0.000774234761747576	0.000773863833696877	0.000772100967129695	0.000770308291852882	0.000769018239187549	0.000767465946827354	0.000764991950316844	0.000763819112342571	0.000763125917750632	0.000761234336036718	0.000759544255060209	0.000757157805162063	0.000756105238717964	0.000754139422551564	0.000752773165533434	0.000750879476486839	0.000748944677482871	0.000746975145184437	0.000746281853118612	0.00074466867980654];                                 
        H = PauCoefMod.*exp(1i*PauCoefFase);
        %Matriz de Igualacion Zero Forcing
        G_cp = H(:);
        %Igualacion de seal
        simb_OFDM = simb_OFDM./repmat(G_cp,1,NTS_OFDM);
       
        
    case 2  % Zero Pading-Igualador Zero Forcing Min Norm
        disp('EL modo de codificacion es ZERO PADING-ZERO FORCING MIN NORM')
        signal_OFDM  = reshape(signal_OFDM ,ceil(N_FFT*(1+CP)),NTS_OFDM);%
        % Estima de Canal Zero Pading M= 64ZP=0.25 1024*8*32 bits T1 = TimeWindow-1.4e-9;
        PauCoefFase = [0.977403470575074	0.428499815525263	-0.113646712751678	-0.655564036509965	-1.1934641052793	-1.72925110883641	-2.26058450874452	-2.79024879367568	-3.3140369477243	-3.83198265013067	-4.35244625591549	-4.86671238209932	-5.37569142494329	-5.88292663530251	-6.38777966279115	-6.89141745966989	-7.39117594030673	-7.88757422506233	-8.37501248888233	-8.86074784298013	-9.34734466679968	-9.83006141616465	-10.3083374373442	-10.7786448787478	-11.2492014761402	-11.7181844094983	-12.1839802637465	-12.6453403789271	-13.099757838168	-13.5525693880987	-14.00629781142	-14.455739104091	-14.8992929022859	-15.3269549671925	-15.7293284124974	-16.1104339347158	-16.4838447324998	-16.8720885025859	-17.3096820073931	-17.7923700196361	-18.3259048502446	-18.91213848477	-19.5387481669309	-20.2186180821796	-20.9216300903138	-21.6360044267859	-22.3413571863988	-23.0265945340022	-23.685127674734	-24.3508463528372	-25.0032888483518	-25.652547672587	-26.2991082681729	-26.9384250996631	-27.5770950268238	-28.2133932142018	-28.8458687732532	-29.473071199386	-30.0970934040947	-30.7182491093666	-31.3377702114291	-31.9518302790241	-32.5665390055841	-33.1689564688698	-33.7764710640922	-34.3779122225202	-34.977084047494	-35.5712274953045	-36.1628247544739	-36.7499801700554	-37.3366810486949	-37.9222464992263	-38.5011072947054	-39.076050742561	-39.6437836929795	-40.2145172184351	-40.7813479276604	-41.3421324308296	-41.8989118999568	-42.4523786946979];        
        PauCoefMod = [0.000749228147799738	0.0007473866568868	0.000747172349314896	0.000745294502310294	0.000742955909305682	0.000741012081323895	0.000738953638403865	0.0007360658847834	0.000741180925770481	0.000734386560914895	0.000733880050785789	0.000731978898391956	0.00072925928255387	0.000728441801351101	0.000726044277397429	0.000725383416775372	0.00072314700449836	0.000719931517405506	0.000719197752224979	0.00071788930781423	0.000716332010283051	0.000714000501894834	0.000711767817143141	0.000710265600820764	0.000708825080730914	0.000705959361380321	0.000703937972625931	0.000702621356122358	0.000700682193268728	0.00069885048234665	0.000695762649463983	0.000694232559266667	0.000692514233119009	0.000681273481309924	0.000661334798280966	0.000646465228353011	0.000647077643976617	0.000664559948162489	0.000691494760976828	0.000712001052298795	0.000729381921328999	0.000732833679440973	0.000722974048237637	0.000715876731151361	0.000722335128429196	0.000733841079667716	0.00075228149948071	0.000771625540590414	0.00079877003990389	0.000783720483856704	0.00078400372842366	0.000782604689814894	0.00078041477509607	0.00078166782284589	0.000780084681874325	0.000781108573664329	0.000779872252460814	0.000776632986657867	0.000776272550256184	0.000777498265209322	0.00077614375168745	0.00077470569134454	0.000770499306865481	0.000770582485306562	0.000771502792394157	0.00077059993666545	0.000767898166474193	0.000769161032654589	0.000767391102914514	0.000763868095125399	0.000763332550523877	0.000761750625703991	0.000761437769166049	0.000763613864973781	0.000757659595546778	0.000756014658673697	0.000754143078879	0.000754507592642499	0.000752078375713509	0.000751061615611114];                                 
        H = PauCoefMod.*exp(1i*PauCoefFase);
        h = ifft(H);
        if length(h)>ceil(N_FFT*CP)
            h = h(1:ceil(N_FFT*CP));
        end
        %%%%%Clculo matriz de igualacin, ver [Muquet02]
        L = length(h);%Orden del canal 
        %H [PxP] matriz Toeplitz triangular inferior de filtrado
        P = ceil(N_FFT*(1+CP));
        h = [h zeros(1,P-L)]';
        H = zeros(P,P);
        H_filt = toeplitz(h);
        for n=1:P
            for m=1:n
                H(n,m)=H_filt(n,m);
            end
        end
        %H_0 [PxN_FFT] es la matriz particin de las 1s N_FFT columnas de la matriz de convolucin 
        H_0 = H(:,1:N_FFT);
        %F es la DFT matrix de tamao [N_FFTxN_FFT]
        F = dftmtx(N_FFT)/sqrt(N_FFT);
        %Matriz de igualacin Zero Forcing
        G_zf = F * pinv(H_0);
        %%%%%Igualacin de seal
        simb_OFDM(1:N_FFT,:) = G_zf * signal_OFDM(1:P,:);
        
        
   case 3   % Zero Pading-Igualador Fast Zero Forcing 
        disp('EL modo de codificacion es ZERO PADING-FAST ZERO FORCING')
        signal_OFDM  = reshape(signal_OFDM ,ceil(N_FFT*(1+CP)),NTS_OFDM);
        % Estima de Canal Zero Pading M=64 ZP=0.25 1024*8*32 bits T1 = TimeWindow-1.4e-9;
        PauCoefFase = [0.977403470575074	0.428499815525263	-0.113646712751678	-0.655564036509965	-1.1934641052793	-1.72925110883641	-2.26058450874452	-2.79024879367568	-3.3140369477243	-3.83198265013067	-4.35244625591549	-4.86671238209932	-5.37569142494329	-5.88292663530251	-6.38777966279115	-6.89141745966989	-7.39117594030673	-7.88757422506233	-8.37501248888233	-8.86074784298013	-9.34734466679968	-9.83006141616465	-10.3083374373442	-10.7786448787478	-11.2492014761402	-11.7181844094983	-12.1839802637465	-12.6453403789271	-13.099757838168	-13.5525693880987	-14.00629781142	-14.455739104091	-14.8992929022859	-15.3269549671925	-15.7293284124974	-16.1104339347158	-16.4838447324998	-16.8720885025859	-17.3096820073931	-17.7923700196361	-18.3259048502446	-18.91213848477	-19.5387481669309	-20.2186180821796	-20.9216300903138	-21.6360044267859	-22.3413571863988	-23.0265945340022	-23.685127674734	-24.3508463528372	-25.0032888483518	-25.652547672587	-26.2991082681729	-26.9384250996631	-27.5770950268238	-28.2133932142018	-28.8458687732532	-29.473071199386	-30.0970934040947	-30.7182491093666	-31.3377702114291	-31.9518302790241	-32.5665390055841	-33.1689564688698	-33.7764710640922	-34.3779122225202	-34.977084047494	-35.5712274953045	-36.1628247544739	-36.7499801700554	-37.3366810486949	-37.9222464992263	-38.5011072947054	-39.076050742561	-39.6437836929795	-40.2145172184351	-40.7813479276604	-41.3421324308296	-41.8989118999568	-42.4523786946979];        
        PauCoefMod = [0.000749228147799738	0.0007473866568868	0.000747172349314896	0.000745294502310294	0.000742955909305682	0.000741012081323895	0.000738953638403865	0.0007360658847834	0.000741180925770481	0.000734386560914895	0.000733880050785789	0.000731978898391956	0.00072925928255387	0.000728441801351101	0.000726044277397429	0.000725383416775372	0.00072314700449836	0.000719931517405506	0.000719197752224979	0.00071788930781423	0.000716332010283051	0.000714000501894834	0.000711767817143141	0.000710265600820764	0.000708825080730914	0.000705959361380321	0.000703937972625931	0.000702621356122358	0.000700682193268728	0.00069885048234665	0.000695762649463983	0.000694232559266667	0.000692514233119009	0.000681273481309924	0.000661334798280966	0.000646465228353011	0.000647077643976617	0.000664559948162489	0.000691494760976828	0.000712001052298795	0.000729381921328999	0.000732833679440973	0.000722974048237637	0.000715876731151361	0.000722335128429196	0.000733841079667716	0.00075228149948071	0.000771625540590414	0.00079877003990389	0.000783720483856704	0.00078400372842366	0.000782604689814894	0.00078041477509607	0.00078166782284589	0.000780084681874325	0.000781108573664329	0.000779872252460814	0.000776632986657867	0.000776272550256184	0.000777498265209322	0.00077614375168745	0.00077470569134454	0.000770499306865481	0.000770582485306562	0.000771502792394157	0.00077059993666545	0.000767898166474193	0.000769161032654589	0.000767391102914514	0.000763868095125399	0.000763332550523877	0.000761750625703991	0.000761437769166049	0.000763613864973781	0.000757659595546778	0.000756014658673697	0.000754143078879	0.000754507592642499	0.000752078375713509	0.000751061615611114];                                 
        H = PauCoefMod.*exp(1i*PauCoefFase);
        h = ifft(H);
        if length(h)>ceil(N_FFT*CP)
            h = h(1:ceil(N_FFT*CP));
        end
        %%%%%Clculo matriz de igualacin, ver [Muquet02]
        L = length(h);%Orden del canal 
        P = ceil(N_FFT*(1+CP));
        %F_p es la DFT matrix de tamao [PxP]
        F_p = dftmtx(P)/sqrt(P);
        %Funcin de transferencia del canal con la que creo matriz diagonal
        H_p=fft(h.',P);
        %Matriz diagonal
        D_p=diag(H_p);
        F_m = dftmtx(N_FFT)/sqrt(N_FFT);
        V_pinv = [F_m zeros(N_FFT,ceil(N_FFT*CP))]*(F_p)';
        %Matriz de igualacin Fast Zero Forcing 
        G_fast_zf = V_pinv*pinv(D_p)*F_p;
        %%%%%Igualacin de seal
        simb_OFDM(1:N_FFT,:) = G_fast_zf * signal_OFDM(1:P,:);

        
    otherwise 
        disp('EL modo de codificacin OFDM no es correcto')
        
end




%%%%DETECCION DE SIMBOLOS

%Subportadoras que transportan datos
simb_QAM(1:Nc,:) = [simb_OFDM(1:Nc/2,:);simb_OFDM(N_FFT-Nc/2+1:N_FFT,:)];

%Serializacin
simb_OFDM = simb_OFDM(:);

%Componentes real e imaginaria de simbolos de datos recibidos
I=real(simb_OFDM);
Q=imag(simb_OFDM);

% Compensacin mediante ceros para figuras de constelacion
I =[I(:).',zeros(1,(TimeWindow*BitRate/BpS)-length(I))]; % Para EVM
Q =[Q(:).',zeros(1,(TimeWindow*BitRate/BpS)-length(Q))]; % Para EVM

%Demodulacion de simbolos y detteccion
hm_trx=modem.qammod('M',2^BpS,'SymbolOrder','Gray','InputType','bit');
hd_rcx=modem.qamdemod(hm_trx);
bits_rx = demodulate(hd_rcx,simb_OFDM);
bits_rx = bits_rx(:).';
% Compensacin mediante ceros de Bit Rate efectivo transmitido
bits_rx = [bits_rx(:).',zeros(1,BpS*length(I)-length(bits_rx))].'; 

disp('Fin Funcion Decoder_OFDM-------------------------------------')	