function[signal_re, signal_im] = Coder_OFDM(prbs,TimeWindow,BitRate,SampleRate,BpS,Nc,N_FFT,CP,Coder)
                    
disp('Inicio Funcion Coder_OFDM-------------------------------------')	

% Coder_OFDM : Codificador de seal electrica OFDM Cyclic Prefix y Zero Pading

%%% PARAMETROS DE ENTRADA
    % prbs: vector de bits de datos
    % TimeWindow: Ventana temporal de simulacion
    % BitRate: Tasa de bits del sistema
    % SampleRate: Tasa de muestreo del sistema
    % BpS: N bits por smbolo del mapeado de subportadoras de datos
    % N_FFT: N total de subportadoras y/o tamao de la FFT
    % Nc: N de subportadoras que transportan datos
    % CP: Longitud intervalo de guarda para Cyclix prefix o Zero Pading respecto el tamo de la FFT
    % Coder: - 1 Cyclix prefix-Zero Forcing
    %        - 2 Zero Pading-Zero Forcing Min Norm
    %        - 3 Zero Pading-Fast Zero Forcing 
%%% PARAMETROS DE SALIDA
    % signal_re: Parte real de seal codificado OFDM temporal    
    % signal_im: Parte imaginaria de seal codificado OFDM temporal
    
%%%% BITS de entrada
bits_tx=prbs(1:TimeWindow*BitRate);
bits_tx=bits_tx(:);

% Clculo de la secuencia de bits efectiva transmitida. BitRateEfectivo=BitRate/(1+CP)
NTS_OFDM=floor(length(bits_tx)/(BpS*ceil(N_FFT*(1+CP)))); %N total de simbolos OFDM transmitidos
NTS_INFO=NTS_OFDM*Nc; %N total de simbolos de datos M-QAM transmmitidos
NTB_INFO=NTS_INFO*BpS; %N total de bits de datos transmitidos
NTS_ZP=(N_FFT-Nc)*NTS_OFDM; %N total de smbolos M-QAM equivalentes a portadoras de guarda
NTS_CP=NTS_OFDM*ceil(N_FFT*CP); %N total de smbolos M-QAM equivalentes a intervalo de guarda
NTS_casados=NTS_INFO+NTS_ZP+NTS_CP; %N total de simbolos M-QAM transmitidos

bits_tx=bits_tx(1:NTB_INFO); %Vector que contiene slo la secuencia de bits a transmitir
bits_tx=bits_tx(:); 


%%%%MODULACION DE SUBPORTADORAS DE DATOS: MAPEADO
% Secuencia de smbolos M-QAM mapeados a partir de bits de datos
hm_trx = modem.qammod('M',2^BpS,'SymbolOrder','Gray','InputType','bit');
simb_QAM = modulate(hm_trx,reshape(bits_tx,NTS_INFO,BpS));
simb_QAM = reshape(simb_QAM,Nc,NTS_OFDM);

% Insercion de bandas de guarda 
if(mod(N_FFT,2)== mod(Nc,2))
     simb_OFDM=[simb_QAM(1:Nc/2,:);zeros(N_FFT-Nc,NTS_OFDM);simb_QAM((Nc/2)+1:Nc,:)]; 
else
    disp('N_FFT y Nc deben ser pares o impares')
end


%%%%PRECODIFICACION IFFT: SEAL TEMPORAL
signal_OFDM = ifft(simb_OFDM,N_FFT)*sqrt(N_FFT);


%%%%CODIFICACION OFDM 
switch Coder
    
    case 1  %Cyclix prefix
        disp('EL modo de codificacion es CYCLIC PREFIX')
        % Adicion de CP a cada simbolo OFDM
        signal_OFDM_out = [signal_OFDM(end-ceil(N_FFT*CP)+1:end,:);signal_OFDM];

    case {2, 3} %Zero Pading
        disp('EL modo de codificacion es ZERO PADING ')
        % Adicion de postfijo de ceros ZP
        signal_OFDM_out = zeros(ceil(N_FFT*(1+CP)),NTS_OFDM);
        signal_OFDM_out(1:N_FFT,:) = signal_OFDM(1:N_FFT,:);
       
    otherwise 
        disp('EL modo de codificacin OFDM no es correcto')
end



%%%%TEST BITRATE
% Adicin de ceros a seal codificada para compensar la perdida de BitRate
% efectivo debido a la insercion de intervalo de guarda
signal_OFDM_out = [(signal_OFDM_out(:).'),zeros(1,((TimeWindow*BitRate/BpS) - NTS_casados))]; 
signal_OFDM_out = signal_OFDM_out(:);



%%%%SOBREMUESTREO
%Insercion de ceros entre muestras previo al sobremuestreo. 
% Interpolacin posterior mediante filtro coseno alzado
signal_up = zeros(floor(TimeWindow*SampleRate),1);
signal_up(1:floor(SampleRate/(BitRate/BpS)):end) = signal_OFDM_out;

%Seal en fase y quadratura
signal_re = zeros(floor(TimeWindow*SampleRate),1);
signal_re(1:floor(SampleRate/(BitRate/BpS))*NTS_casados) = real(signal_up(1:floor(SampleRate/(BitRate/BpS))*NTS_casados));
signal_im = zeros(floor(TimeWindow*SampleRate),1);
signal_im(1:floor(SampleRate/(BitRate/BpS))*NTS_casados) = imag(signal_up(1:floor(SampleRate/(BitRate/BpS))*NTS_casados));


disp('fin Funcion Coder_OFDM-------------------------------------')	